/* 

                          Firewall Builder

                 Copyright (C) 2001 Vadim Zaliva, Vadim Kurland

  Author:  Vadim Zaliva lord@crocodile.org

  $Id: FWObjectDatabase.hh,v 1.4 2001/10/05 04:38:47 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#ifndef __FWOBJECTDATABASE_HH_FLAG__
#define __FWOBJECTDATABASE_HH_FLAG__

#include "FWObject.hh"
#include "FWException.hh"
#include "ThreadTools.hh"
#include "XMLTools.hh"

namespace libfwbuilder
{

/**
 * Database of objects.
 */
class FWObjectDatabase : public FWObject
{
    protected:

    static long               IDcounter     ;
//    static Mutex              IDmutex       ;
    static const string       DTD_FILE_NAME ;

    string data_file;

    FWObjectDatabase();
    virtual ~FWObjectDatabase() {};
    
    public:

    DECLARE_FWOBJECT_SUBTYPE(FWObjectDatabase);
    
    // --- XML import/export ---
    
    /**
     * This is the main "Create" method:
     * it creates instance of FWObject of given type
     */
    virtual FWObject *create(const string &type);

    /**
     * Creates instance of FWObject using its XML representation
     */
    virtual FWObject *createFromXML(xmlNodePtr data);

    virtual xmlNodePtr toXML(xmlNodePtr parent);
    
    string generateUniqueId();
    
    // --- Load/Save ---
    
    virtual void saveFile       ( const string &filename) throw(FWException); 
    virtual void saveXML        ( xmlDocPtr             ) throw(FWException); 
    virtual void load           ( const string &filename, XMLTools::UpgradePredicate *upgrade) throw(FWException);

    void setFileName (const string &filename);
    const string& getFileName ();
    const string  getFileDir ();

    // --- Standard objects ---

    struct StandardObjects
    {
	string     rootId;

        string     AnyNetworkId;
        string     AnyIPServiceId;
        string     AnyIntervalId;
        
        string     ObjectsId;
        string     HostsId;      
        string     NetworksId;    
        string     ObjectGroupsId;
        
        string     ServicesId;
        string     IPServicesId;
        string     ICMPServicesId;
        string     UDPServicesId;
        string     TCPServicesId;
        string     ServiceGroupsId;
        string     CustomServicesId;

        string     TimeId;
        
        string     FirewallsId;
    };

    struct StandardObjects std;

    // singelton

    static FWObjectDatabase  *db;

};

}

#endif

